
// shorthand for common redundant references
var Ci = Components.interfaces;
var Cc = Components.classes;
var console = Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService);
var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);

// namespace
var f = new Object();

// set host string for password manager
f.hostString = "urn:foxylicious:username";

// for pref management
f.prefs = false;
try{
    f.prefs = Cc["@mozilla.org/preferences;1"].getService(Ci.nsIPrefBranch);
}catch(e){
	Components.utils.reportError(e);
}
// globalize b/c ns not used everywhere yet
var prefs = f.prefs;

function getPref(strPrefName)
{
  var pref = false;
  var type = f.prefs.getPrefType(strPrefName);
  
  try {
    switch(type) {
      case f.prefs.PREF_STRING:
        pref = f.prefs.getCharPref(strPrefName);
        break;
      case f.prefs.PREF_INT:
        pref = f.prefs.getIntPref(strPrefName);
        break;
      case f.prefs.PREF_BOOL:
        pref = f.prefs.getBoolPref(strPrefName);
        break;
      default:
        return false;
    }    
  }
  catch(e) {
    return false;
  }
  
  if(pref && pref != "")
    return pref;
  
  return false;
}

function logMessage(message) 
{
  if(getPref('foxylicious.debug') && console)
  {
    console.logStringMessage("Foxylicious: " + message);
  }
}

function log(str) {
	if(!getPref('foxylicious.debug')){return;}
	
	console.logStringMessage(str);
	if (arguments.length > 1){
		var dumpObj = arguments[1];
		for (key in dumpObj){
			console.logStringMessage(" " + key + ":" + dumpObj[key]);
		}
	}
}

function error(e)
{
	Components.utils.reportError(e);
}

// for strings used from script
var bs = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
f.stringBundle = bs.createBundle("chrome://foxylicious/locale/foxylicious.properties");

function initDelicious()
{
  // delicious
  del = new Delicious();
  del.async = false;
  del.debug_flag = getPref("foxylicious.debug");
  
  // set del.icio.us credentials
  var username = getPref('foxylicious.username');
  var password = getPassword(f.hostString, username);

  // if no encrypted password exists, try to get unencrypted one
  if(!password)
    {
      password = getPref('foxylicious.password');
      // if unencrypted password exists, store it encrypted and remove unencrypted
      if(password)
      {
        // attempt to save encrypted
        if(savePassword(f.hostString, username, password))
        {
          // remove unencrypted
          f.prefs.setCharPref('foxylicious.password', false);
        }
      }
    }

  // check for credentials
  if(username == "" || password == "")
	return;

  del.setCredentials(username, password);
}

/*
	Core function called when "update bookmarks" button 
	is pressed in settings dialog.
*/
function exportBookmarksToDeliciousFF32()
{
		initDelicious();
	var exportFolder=	 getPref('foxylicious.exportFolder');
	var recurseFolders=	 getPref('foxylicious.recurseFolders');
	var useFolderNamesAsTag =	 getPref('foxylicious.folderNameTag');
	var exportAsPrivate = 	getPref('foxylicious.exportAsPrivate');
	if(!exportFolder)
	{
		alert('You must select a bookmark folder to export!');
	}
/*	else if(exportFolder == folderLocation)
	{
		alert('You cannot export your local del.icio.us bookmarks into del.icio.us. That is just silly.');
	} */ 
	else
	{
		//var useFolders = document.getElementById('folderNameTag').checked;
		//exportBookmarksToDeliciousFF3(exportFolder, useFolders);
		exportBookmarksToDeliciousFF3(exportFolder,recurseFolders,useFolderNamesAsTag,exportAsPrivate);
	}
}
function integrateDeliciousBookmarks()
{
  var bmWrapper = new BookmarksWrapper();
  var result = false;
  var mCaption=document.getElementById('cap1Id');
	
	mCaption.label="Fetching bookmarks from delicious!";
  
  // get folder location
  var delFolderID = getPref('foxylicious.folderLocation');
  if(!delFolderID)
  {
    try {
      alert(f.stringBundle.GetStringFromName("chooseFolder"));
    } catch(ex) {
		error(ex);
    }
    return false;
  }
  var delFolder = delFolderID;//bmWrapper.getBookmarksFolder();
  // get bundles prefs
  var useBundles = getPref('foxylicious.useBundles');
  var useOnlyBundles = getPref('foxylicious.useOnlyBundles');

  // if bundles are enabled, get and process bundles
  if(useBundles)
  {
	var bundles = new Array();
    var bundleData = del.get_all_bundles();
    for(var i in bundleData)
    {
      var row = bundleData[i];
      bundles[ row.name ] = " " + row.tags + " ";
    }
  }

  // if update is needed, get all posts
  result = del.get_all_posts();
  //logMessage( del.req.responseText );

  // update bookmarks w/ posts
  logMessage('Del.icio.us results: ' + result.length);
  
  if(result.length == 0)
    return;

  // empty current delicious folder
  var fullSync = getPref('foxylicious.fullSync');
  if(fullSync && confirm(f.stringBundle.GetStringFromName("confirmDelete")))
  {
    bmWrapper.deleteFolderContents(delFolder);
    alert(f.stringBundle.GetStringFromName("postDelete"));
  }
  
  var str = "";
  var url;
  var tags;
  var i; // results
  var k; // tags
  var l; // subtags
  var tagFolder;
  var delimiter = getPref('foxylicious.delimiter');
  var subTags;
  var curFolder;
  var init = true;
  var useKeywords = false;
  var keyword = "";
  var extended = "";

  var createHierarchy = getPref('foxylicious.hierarchyFromTags');
  var normalizeTags = getPref('foxylicious.normalizeTags');
  
  // loop thru result set
  var m=0;
  for( i in result )
 // for (var i=0;i<result.length;i++)
  {
	  sleep(200);
  	m++;
    url = result[i].href;
    mCaption.label=parseInt(m)+"/"+parseInt(result.length/2)+" - "+url;
    // get extended
    extended = (result[i]['extended'] != null) ? result[i]['extended'] : "";

    // get tags, process
    tags = result[i]['tag'].split(" ");
    
    // if bm doesn't have any tags, bookmark it in root folder and continue
    if(   createHierarchy && (tags.length == 0)) 
    {
		continue;
	}

    // normalize tags to lower-case
    if(normalizeTags)
    {
      for( k in tags )
      {
        tags[k] = tags[k].toLowerCase();
      }
    }
          
    // loop through tags
    if (createHierarchy) 
    {
    for( k in tags )
    {
      // begin bundles support
      if(useBundles)
      {
        var bundleFolder;
        var tagFolder;

        // loop thru bundles
        for(var bundleName in bundles)
        {
          // if tag is in this bundle
          if(bundles[bundleName].indexOf(" "+tags[k]+" ") > -1)
          {
            // if bundle doesn't have a folder in del folder
            if(!bmWrapper.hasChildFolder(delFolder, bundleName))
            {
              // create /del/bundle folder
              bundleFolder = bmWrapper.createFolder(bundleName, delFolder);
            }
            else
            {
              bundleFolder = bmWrapper.getChildFolder(delFolder, bundleName);
            }

            // if tag doesn't have a folder in /bundles/bundle
            if(!bmWrapper.hasChildFolder(bundleFolder, tags[k]))
            {
              // create /bundles/bundle/tag
              tagFolder = bmWrapper.createFolder(tags[k], bundleFolder);
            }
            else
            {
              tagFolder = bmWrapper.getChildFolder(bundleFolder, tags[k]);
            }

            // if url is not in /bundles/bundle/tag
            if(!bmWrapper.isBookmarkedResource(result[i].href, tagFolder))
			{
              // create bookmark
				keyword = [tags[k]];
				bmWrapper.createBookmark(tagFolder, ios.newURI(result[i].href, null, null), -1, result[i].description, keyword);
            }
          }
        }
      }
      
      if(!useOnlyBundles)
      {
        // normal processing: create /bookmarks/tag/link
        // only do regular processing if delimiters are not turned on
        // or the link doesn't contain the delimiter
        if(!delimiter || tags[k].indexOf(delimiter) < 1)
        {
            // create a new folder for this tag if there's not one already
            if(!bmWrapper.hasChildFolder(delFolder, tags[k]))
            {
                tagFolder = bmWrapper.createFolder(tags[k], delFolder);
            }
            // else get existing folder
            else
            {
                tagFolder = bmWrapper.getChildFolder(delFolder, tags[k]);
            }
            
            // create new bookmark if one does not already exist in this tag's folder
            if(!bmWrapper.isBookmarkedResource(result[i].href, tagFolder))
            {
				keyword = [tags[k]];
				bmWrapper.createBookmark(tagFolder, ios.newURI(result[i].href, null, null), -1, result[i].description, keyword);
            }
        }
        
        // create tag hierarchy
        /*
        begin tag combination hierarchy
        this creates subfolders in the current tag's folder
        for each other tag this url has, and places the url in all of them.
        */
        if(createHierarchy)
        {
          for(l in tags)
          {
            if(tags[l] != tags[k])
            {
              // create a new subfolder for this tag if there's not one already
              if(!bmWrapper.hasChildFolder(tagFolder, tags[l]))
              {
                subTagFolder = bmWrapper.createFolder(tags[l], tagFolder);
              }
              // else get existing folder
              else
              {
                subTagFolder = bmWrapper.getChildFolder(tagFolder, tags[l]);
              }
              
              // add bookmark to subfolder, if one does not already exist
              if(!bmWrapper.isBookmarkedResource(result[i].href, subTagFolder))
              {
				keyword = [tags[l]];
				bmWrapper.createBookmark(subTagFolder, ios.newURI(result[i].href, null, null), -1, result[i].description, keyword);
              }
            }
          }
        }

        // begin delimiter support
        
        // set initial folder for this url to the delicious folder
        tagFolder = delFolder;
        
        // look for delimiter, and split on it
        if(delimiter && tags[k].indexOf(delimiter) > 0)
        {
          subTags = tags[k].split(delimiter);
          
          // create folder structure, and get the entry's final resting place
          for(l in subTags)
          {
            // if there's no folder already in the tagFolder for this tag
            if(!bmWrapper.hasChildFolder(tagFolder, subTags[l]))
            {
              // create a new folder
              tagFolder = bmWrapper.createFolder(subTags[l], tagFolder);
            }
            // else get existing folder
            else
            {
              tagFolder = bmWrapper.getChildFolder(tagFolder, subTags[l]);
            }
          }
          
          // create bookmark if one does not already exist in the tags folder
          if(!bmWrapper.isBookmarkedResource(result[i].href, tagFolder))
          {
			keyword = [subTags[l]];
            bmWrapper.createBookmark(tagFolder, ios.newURI(result[i].href, null, null), -1, result[i].description, keyword);
          }
        } // end of delimiter support
      } // end of !useOnlyBundles block
    } // end of tag loop
    } // end of createHirachy
    else
    {
    	 // create bookmark if one does not already exist in the tags folder
          if(!bmWrapper.isBookmarkedResource(result[i].href, delFolderID))
          {
			//keyword =  [tags[0]];
            bmWrapper.createBookmark(delFolderID, ios.newURI(result[i].href, null, null), -1, result[i].description, tags);
          }
    	
    }
  } // end of result (link) loop
  
  // sort by name
  //bmWrapper.sortFolder( delFolder , false , true , true , false, false , "Name");
  
  // set lastUpdated
	if (del.rootNodeAttributes){
  		f.prefs.setCharPref('foxylicious.lastUpdatedRemote', del.rootNodeAttributes['update'] );
	}
}

function delAddPost(uri,title,tags,desc,count,noShare)
{
	//alert("before");
//var win=new Window();//	openWindow("http://example.tld/zzz.xul", "dlg", "", "pizza", 6.98);


	sleep(200);
	//sleep(200);
	var mProgress = document.getElementById("prog1Id");
	
	mProgress.value =count/BookmarksWrapper.bookmarkCount*100+'';
	var mCaption=document.getElementById('cap1Id');
	
	mCaption.label=count+'/'+BookmarksWrapper.bookmarkCount+' - '+title;
	//mProgress.caption=title;
//	document.getElementById('prog1Id').value++;//=document.getElementById('prog1Id').value+1;
	//document.getElementById('cap1Id').value=title;
	// alert("after");
	//win.close();
	 del.add_post(
         uri,
          title,
         desc,
          tags,
          "", //,bmWrapper.getBMDate(temptarget)
          noShare
        );
       
}
function sleep(aWait) {
  var timer = { timeup: false };

  var interval = window.setInterval(function(){
    timer.timeup = true;
  }, aWait);

  var thread = Cc["@mozilla.org/thread-manager;1"].getService().mainThread;
  while(!timer.timeup){
    thread.processNextEvent(true);
  }
  window.clearInterval(interval);
}

function exportBookmarksToDeliciousFF3(exportFolderId,useFolders,useFolderNamesAsTag,exportAsPrivate)
{
  logMessage('Exporting bookmarks');
  var bmWrapper = new BookmarksWrapper();
  bmWrapper.exportBookmarks(exportFolderId,useFolders,false,delAddPost,useFolderNamesAsTag,exportAsPrivate);
	
}

function exportBookmarksToDelicious(target, useFolders)
{
  logMessage('Exporting bookmarks');
  var bmWrapper = new BookmarksWrapper();
  var result = false;
  
  // get delicious folder location
  var delFolder = f.prefs.getCharPref('foxylicious.folderLocation', "");
  delFolder = RDF.GetResource(delFolder);
  
  // get bookmarks root
  var bmRoot = RDF.GetResource("NC:BookmarksRoot");
  
  var tags = "";
  
  var folders = [target];
  while(folders.length > 0)
  {
    var folder = folders.shift();
    folder.QueryInterface(Components.interfaces.nsIRDFResource);
    bmWrapper.Container.Init(bmWrapper.DataSource, folder);
    var elements = bmWrapper.Container.GetElements();
    while(elements.hasMoreElements())
    {
      var temptarget = elements.getNext();
      if(bmWrapper.RDFCUtils.IsContainer(bmWrapper.DataSource,temptarget))
      {
        // TODO: check if folder is folderLocation. if so, do not add it
        
        folders.push(temptarget);
        logMessage('adding folder: ' + bmWrapper.getBMTitle(temptarget) );
      }
      else
      {
        logMessage('Exporting ' + bmWrapper.getBMTitle(temptarget));
        
        // use folder name for tag?
        if(useFolders)
        {
          tags = bmWrapper.getBMTitle(folder);
          var arr = tags.split(' ');
          tags = '';
          for(var i in arr)
          {
            tags += arr[i];
          }
        }
        
        // sleep for a second
        //pauseDialog(1000, "adding link");
        
        del.add_post(
          bmWrapper.getBMURL(temptarget),
          bmWrapper.getBMTitle(temptarget),
          bmWrapper.getBMDescription(temptarget),
          tags,
          "" //,bmWrapper.getBMDate(temptarget)
        );
      }
    }
  }
}

/*
  &dt= datestamp for post, format "CCYY-MM-DDThh:mm:ssZ"
makes a post to delicious. 
the datestamp requires a LITERAL "T" and "Z" like in ISO8601 at
http://www.cl.cam.ac.uk/~mgk25/iso-time.html. for example:
"1984-09-01T14:21:31Z"
*/

Date.prototype.toISO8601String = function()
{
  var str;
  
  // date
  var year = this.getUTCFullYear().toString();
    var month = (this.getUTCMonth()+1).toString();
    month = month.length < 2 ? ("0" + month) :  month;
    
    var day = this.getUTCDate().toString();
    day = day.length < 2 ? ("0" + day) : day;
    
    var hours = this.getUTCHours().toString();
    hours = hours.length < 2 ? ("0" + hours) : hours;
    
    var minutes = this.getUTCMinutes().toString();
    minutes = minutes.length < 2 ? ("0" + minutes) : minutes;
    
    var seconds = this.getUTCSeconds().toString();
    seconds = seconds.length < 2 ? ("0" + seconds) : seconds;
    
    return year + "-" + month + "-" + day + "T" + hours + ":" + minutes + ":" + seconds + "Z";
}

Date.prototype.fromISO8601String = function(str)
{
  //logMessage('fromISOString: ' + str);
  if(!str || str == "")
  {
    return false;
  }
  
  str = str.replace("Z","");
  var parts = str.split("T");
  var dateArr = parts[0].split("-");
  var timeArr = parts[1].split(":");
  
  this.setUTCFullYear(dateArr[0]);
  this.setUTCMonth(dateArr[1]-1);
  this.setUTCDate(dateArr[2]);
  this.setUTCHours(timeArr[0]);
  this.setUTCMinutes(timeArr[1]);
  this.setUTCSeconds(timeArr[2]);
  var dstr = "y: " + dateArr[0] 
  + ", m: " + dateArr[1]
  + ", d: " + dateArr[2]
  + ", h: " + timeArr[0]
  + ", m: " + timeArr[1]
  + ", s: " + timeArr[2];
  //logMessage('fromISOString: ' + dstr);
}

// http://www.faqts.com/knowledge_base/view.phtml/aid/1602
function pauseDialog(numberMillis, str) {
    var dialogScript = 
       'window.setTimeout(' +
       ' function () { window.close(); }, ' + numberMillis + ');';
    var result = 
     openDialog(
       'javascript:document.writeln(' +
        '"<script>' + dialogScript + '<' + '/script>")',
       'pauseDialog', 'modal=1,width=100,height=100');
 }

// save password
function savePassword(host, user, pass)
{
    // get component
    var pm = false;
    try {
		pm = Cc["@mozilla.org/login-manager;1"].getService(Ci.nsILoginManager);
    } catch (e) {
        logMessage("failed to get password manager\n");
        return false;
    }

	var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
	                                             Ci.nsILoginInfo, "init");
	var loginInfo = null;
	try {
		var logins = pm.findLogins({}, host, null, f.hostString);
		for (var i = 0; i < logins.length; i++) {
			if (logins[i].username == user) {
				loginInfo = logins[i];
		        break;
		    }
		}
		
		if (loginInfo == null) {
			loginInfo = new nsLoginInfo(host, null, f.hostString, user, pass, "", "");
			pm.addLogin(loginInfo);
			return true;
		}
		
		pm.removeLogin(loginInfo);
		
		loginInfo = new nsLoginInfo(host, null, f.hostString, user, pass, "", "");
		pm.addLogin(loginInfo);
	} catch (ex) {
		logMessage("savePassword() failed\n" + ex);
		return false;
	}
	
    return true;
}

function getPassword(host, user) 
{
	var pm = null;
    try {
        pm = Cc["@mozilla.org/login-manager;1"].getService(Ci.nsILoginManager);
    } catch (e) {
        logMessage("getPassword() failed\n" + e);
        return false;
    }

	try {
		// Find users for the given parameters
		var logins = pm.findLogins({}, host, null, f.hostString);
		
		// Find user from returned array of nsILoginInfo objects
		for (var i = 0; i < logins.length; i++) {
			if (logins[i].username == user) {
				return logins[i].password;
			}
		}
	} catch (ex) {
		Components.utils.reportError(ex);
		logMessage("getPassword() failed\n" + ex);
		return false;
	}
	
    return false;
}
