// run init() a couple of seconds after the browser starts up
// (b/c got unreliable results when doing it right away)
window.addEventListener(
  "load",
   function(){
	initDelicious();
	init();
	},
  true
);

function init()
{
  var bmWrapper = new BookmarksWrapper();
  var folderLocation = getPref('foxylicious.folderLocation');

  // create and set default folder if none set already
  if(!folderLocation)
  {
	var delFolder = bmWrapper.getBookmarksFolder();
	
	// if default folder doesn't already exist, create it now
	if (!delFolder)
	{
		delFolder = bmWrapper.createFolder(bmWrapper.DEFAULT_FOLDER_NAME);
		log('running foxylicious for the first time, so creating a del.icio.us folder to use as default location.');
	}
	
    // store in prefs
    f.prefs.setCharPref('foxylicious.folderLocation', delFolder);
  }

  // start update-checking cycle
  foxyCheckForUpdates();
}

/*
	Once called, this function checks for updates, then runs itself every 5 minutes.
	It will only update again if the date has changed to a different day,  this allows 
	for users who never shut down their browser to continually get updates

	note: we update the lastUpdated pref if we call /update or we do an update
	note: we set the lastUpdatedRemote pref only after an update, and with the value 
	of the last remote update value
*/
function foxyCheckForUpdates()
{
  logMessage('foxyCheckForUpdates()');
  
  // check to see if daily updates are enabled
  var updateDaily = getPref('foxylicious.updateDaily');
  
  if (!updateDaily) {
    logMessage('Daily updates are not enabled');
    window.setTimeout('foxyCheckForUpdates()', 1800000);
    return;
  }

  logMessage("Daily update is enabled.");
    
  var todaysDate = new Date();
  var runUpdate = false;
  
  // get the local last-updated pref
  var lastUpdated = getPref('foxylicious.lastUpdated');
  if(lastUpdated)
  {
    var lastUpdatedDate = new Date();
    lastUpdatedDate.fromISO8601String( lastUpdated );
    
    // if the pref day is different than today
    if(todaysDate.getUTCDate() != lastUpdatedDate.getUTCDate())
    {
      logMessage('Day has changed since last update attempt (' + todaysDate.getUTCDate() 
				+ ' != ' + lastUpdatedDate.getUTCDate() + '), so calling del.icio.us/update...');
      
      // get lastUpdatedRemote pref
      var lastUpdatedRemotePref = getPref('foxylicious.lastUpdatedRemote');
      
      if(lastUpdatedRemotePref)
      {
        var lastUpdatedRemotePrefDate = new Date();
        lastUpdatedRemotePrefDate.fromISO8601String( lastUpdatedRemotePref );
      
        // get last updated date from delicious
        var lastUpdatedDateRemote = false;
        var lastUpdatedRemote = del.update();
        
        // if we got a result
        if(lastUpdatedRemote)
        {
          // set pref indicating last time we called update() api
          f.prefs.setCharPref('foxylicious.lastUpdated', todaysDate.toISO8601String() );
          
          lastUpdatedDateRemote = new Date();
          lastUpdatedDateRemote.fromISO8601String( lastUpdatedRemote );
        
          // if local date is older than remote date, then update
          if(lastUpdatedDateRemote > lastUpdatedRemotePrefDate)
          {
            logMessage("The remote last-updated date (" + lastUpdatedDateRemote 
						+ ") is newer than the remote last-updated date pref (" 
						+ lastUpdatedRemotePrefDate + "), so running update.");
            runUpdate = true;
          }
          else
          {
            logMessage("The remote last-updated date (" + lastUpdatedDateRemote 
						+ ") is not different than the remote last-updated date pref (" 
						+ lastUpdatedRemotePrefDate + "), so not running update.");
          }
        }
        // no remote result
        else
        {
          logMessage("Unable to determine the remote last-updated date, so running update.");
          runUpdate = true;
        }
      }
      else
      {
        logMessage("No lastUpdatedRemote pref, or empty, so running update.");
        runUpdate = true;
      }
    }
    else
    {
      logMessage("The lastUpdated pref is same as today, so not updating (" 
				+ lastUpdatedDate.getUTCDate() + " == " + todaysDate.getUTCDate() + ")");
    }
  }
  // run update if no pref found
  else
  {
    runUpdate = true;
    logMessage("Empty or not defined lastUpdated pref, so running update.");
    
    // set pref indicating last time we updated locally
    f.prefs.setCharPref('foxylicious.lastUpdated', todaysDate.toISO8601String() );
  }
  
  if(runUpdate)
    integrateDeliciousBookmarks();
  
  // run again in 30 minutes
  // NOTE: doing this even if daily updates aren't enabled b/c user might change the pref
  // XXXDietrich implement pref observer instead
  window.setTimeout('foxyCheckForUpdates()', 1800000);
}

// load settings from tools menu
function loadDeliciousOptions() {
  var dialogURL = "chrome://foxylicious/content/settings.xul";
  window.openDialog(dialogURL, "", "chrome,modal,close,resizable=yes");
}

function addToDelicious() {  
  var win = window.content;
  var context = gContextMenu;
  var src = context.target;
  var doc = src.ownerDocument ? src.ownerDocument : win.document;
  
  var username = getPref( 'foxylicious.username' );
  if(username)
  {
    try {
		var postURL = getPref("foxylicious.postURL");
		// fix url for existing extension installations
		if (postURL.indexOf("delicious.com") < 0)
		{
			f.prefs.setCharPref("foxylicious.postURL", "http://delicious.com/save?v=2&noui=yes&jump=close&url=%location%&title=%title%");
			postURL = getPref("foxylicious.postURL");
		}
      	
        postURL.replace("%user%", username).
        replace("%location%", encodeURIComponent(doc.location)).
        replace("%title%", encodeURIComponent(doc.title));
      	window.open(postURL, 'delicious', 'toolbar=no,width=700,height=370');
    } catch(ex) {
		Components.utils.reportError(ex);
		alert(ex);
    }
  }
  else
  {
    alert("No username! You must first configure Foxylicious via the Tools menu!");
  }
}
